;==============================================================================
; Generate the "xxx Management.htm" pages
; from the "xxx TOC.hhc"

; Input and output directories are read from 'txt2htm.ini'
; The directory should contain either keywords or functions; not both!
;
; "xxx Management" in "UDF TOC.hhc" must be unique unless reference to #include file will be wrong
;
; 19 Jan 2009 - jpm
;			- Initial
; 12 Mar 2013 - jpm
;			- Reorg to have same source for AutoItX generation
; 13 Jul 2013 - jos
;           - Changed the path for include to support building from DOCS directory
; xx XXX 2014 - Guinness
;			- Html5 page support
; 24 Jul 2015 - jpm
;			- Support in WinAPIEx Reference of pages moved from WinAPI Reference
;==============================================================================

#include "OutputLib.au3"
#include <Array.au3>
#include <Constants.au3>

Opt("TrayIconDebug", 1)

OnAutoItExitRegister("OnQuit") ; ### when run in standalone
_OutputWindowCreate() ; ### when run in standalone

Global $g_iReGen_AutoIt, $g_iReGen_UDFs, $g_iReGen_AutoItX = 0
$g_iReGen_AutoIt = StringInStr($CmdLineRaw, "/AutoIt")
$g_iReGen_UDFs = StringInStr($CmdLineRaw, "/UDFs")

If FileExists(@WorkingDir & "\AutoItX.hhp") Then
	$g_iReGen_AutoItX = 1
Else
	If $g_iReGen_AutoIt = 0 And $g_iReGen_UDFs = 0 Then
		; reGenerate AutoIt and UDFs
		$g_iReGen_AutoIt = 1
		$g_iReGen_UDFs = 1
	EndIf
EndIf

Global Const $TXT2HTM_INI = @WorkingDir & "\txt2htm.ini"
Global $g_hOut ;handle to the output file
Global $g_hIn ;handle to current input file
Global $g_sOUTPUT_DIR
Global $g_sRefType, $g_sRefTypeS
Global $g_sFTOC, $g_sFNAME, $g_sNAME, $g_iNestedMgt, $g_ahOut[4], $g_anPrevFuncname_0[4], $g_aFuncname[500], $g_aIncludeAU3[100][3]

If $g_iReGen_AutoIt Then
	$g_sOUTPUT_DIR = IniRead($TXT2HTM_INI, "Output", "functions", "ERR")
	$g_sFTOC = ""
	$g_iNestedMgt = 0

	$g_sRefType = "Function"
	$g_sRefTypeS = "Function"
	$g_hIn = FileOpen("AutoIt3 TOC.hhc") ; Input mode
	GetFirstManagement()
	genFiles()
	FileClose($g_hIn)
EndIf

If $g_iReGen_UDFs Then
	$g_sOUTPUT_DIR = IniRead($TXT2HTM_INI, "Output", "libfunctions", "ERR")
	$g_sFTOC = ""
	$g_iNestedMgt = 0
	SetIncludeAU3()

	$g_sRefType = "User Defined Function"
	$g_sRefTypeS = "User Defined Functions"
	$g_hIn = FileOpen("UDFs3 TOC.hhc") ; Input mode
	GetFirstManagement()
	genFiles()
	FileClose($g_hIn)
EndIf

If $g_iReGen_AutoItX Then
	$g_sOUTPUT_DIR = IniRead($TXT2HTM_INI, "Output", "methods", "ERR")
	$g_sFTOC = ""
	$g_iNestedMgt = 0

	$g_sRefType = "Method"
	$g_sRefTypeS = "Methods"
	$g_hIn = FileOpen("AutoItX TOC.hhc") ; Input mode
	GetFirstManagement()
	genFiles()
	FileClose($g_hIn)
EndIf

Exit

Func OnQuit()
	_OutputWaitClosed() ; ### when run in standalone
EndFunc   ;==>OnQuit

Func putHeader()
	Local $sIncludeName, $iExtended
	put('<!DOCTYPE html>')
	put('<html>')
	put('<head>')
	put('  <title>' & $g_sRefType & "s" & '</title>')
	put('  <meta charset="utf-8">')
	If $g_iReGen_AutoItX Then
		put('  <link href="../../css/default.css" rel="stylesheet">')
	Else
		put('  <link href="../css/default.css" rel="stylesheet">')
	EndIf
	put('</head>')
	put('')
	put('<body>')
	put('<h1>' & $g_sNAME & ' ' & StringLower($g_sRefType) & 's Reference</h1>')
	If $g_iReGen_AutoItX Then
		put('<p>Below is a complete list of the ' & StringLower($g_sRefType) & 's available in AutoItX.')
	Else
		put('<p>Below is a complete list of the ' & StringLower($g_sRefType) & 's available in AutoIt.')
	EndIf
	put('Click on a ' & StringLower($g_sRefType) & ' name for a detailed description.</p>')
	If $g_sRefType = "User Defined Function" Then
		$sIncludeName = GetIncludeAU3($g_sNAME)
		$iExtended = @extended
		put('<p>When using them you need to add a <strong>#include &lt;' & $sIncludeName & '.au3&gt;</strong>.')
		If $iExtended Then
			put('<br>Instead UDF flagged with (*) need to add a <strong>#include &lt;WinAPI.au3&gt;</strong>')
		EndIf
		put('</p>')
	Else
		If StringInStr($g_sNAME, "GUI") = 1 Then
			put('<p>See <a href="../guiref/GUIConstants.htm">Gui Constants include files</a> if you need to use the related controls Constants .</p>')
		EndIf
	EndIf
	put('<p>&nbsp;</p>')
	put('')
	put('<table>')
	put('<tr>')
	put('  <th style="width:25%">' & $g_sRefType & '</th>')
	put('  <th style="width:75%">Description</th>')
	put('</tr>')
EndFunc   ;==>putHeader

Func GetIncludeAU3($sName)
	$sName = StringReplace($sName, " Reference", "")
	For $i = 1 To $g_aIncludeAU3[0][0]
		If $g_aIncludeAU3[$i][0] = $sName Then
			If $g_aIncludeAU3[$i][2] Then SetExtended(1)
			$sName = $g_aIncludeAU3[$i][1]
			ExitLoop
		EndIf
	Next
	Return $sName
EndFunc   ;==>GetIncludeAU3

Func SetIncludeAU3()
	Local $sINPUT_DIR = IniRead($TXT2HTM_INI, "Input", "libfunctions", "ERR")
	Local $hToc = FileOpen($sINPUT_DIR & "Categories.toc") ; Input mode
	Local $sLine, $aSplit, $sPrevMgt = ""
	$g_aIncludeAU3[0][0] = 0

	While 1
		$sLine = FileReadLine($hToc)
		If @error Then ExitLoop
		If StringInStr($sLine, ":") = 0 Then ContinueLoop
		If (StringInStr($sLine, "WinAPIEx") = 1) And StringInStr($sLine, "WinAPI:") Then
			; check if mix include in the same management page
			If StringInStr($sLine, $sPrevMgt) Then $g_aIncludeAU3[$g_aIncludeAU3[0][0]][2] = "WinAPI"
			ContinueLoop
		EndIf
		$aSplit = StringSplit($sLine, "|")
		If $aSplit[$aSplit[0] - 1] = $sPrevMgt Then ContinueLoop
		$sPrevMgt = $aSplit[$aSplit[0] - 1]
		$g_aIncludeAU3[0][0] += 1
		$g_aIncludeAU3[$g_aIncludeAU3[0][0]][0] = StringReplace(StringReplace($sPrevMgt, " Management", ""), "&", "&amp;")
		$aSplit = StringSplit($aSplit[$aSplit[0]], ":")
		$g_aIncludeAU3[$g_aIncludeAU3[0][0]][1] = $aSplit[1]
		$g_aIncludeAU3[$g_aIncludeAU3[0][0]][2] = ""
	WEnd

	FileClose($hToc)
EndFunc   ;==>SetIncludeAU3

Func PutEntry()
	Local $sFuncname = GetFuncName($g_sFNAME)
	If $sFuncname = "" Then Return
	$g_aFuncname[0] += 1
	$g_aFuncname[$g_aFuncname[0]] = $g_sFNAME
EndFunc   ;==>PutEntry

Func GetIncludeName($sNFname, $iLine = 15)
	Local $sIncludeName = FileReadLine($g_sOUTPUT_DIR & $sNFname, $iLine)
	$sIncludeName = StringRegExp($sIncludeName, '#include +&lt;(.*?)&gt', 1)
	If @error Then
		If $iLine = 15 Then Return GetIncludeName($sNFname, 16)
		Return ""
	EndIf
	Return $sIncludeName[0]
EndFunc   ;==>GetIncludeName

Func GetFuncDesc($sNFname, $iLine = 13)
	Local $vFuncdesc = FileReadLine($g_sOUTPUT_DIR & $sNFname, $iLine)
	$vFuncdesc = StringRegExp($vFuncdesc, '<(?i)p .*?>(.*?)<(/(?i)p|br)>', 1)
	If @error Then
		If $iLine = 13 Then Return GetFuncDesc($sNFname, 14)
		Return "&nbsp;"
	EndIf
	Return $vFuncdesc[0]
EndFunc   ;==>GetFuncDesc

Func GetFuncName($sNFname)
	Local $sLine = FileReadLine($g_sOUTPUT_DIR & $sNFname, 4)
	Local $sFuncname = StringRegExp($sLine, '<title>.*? +(.*?) *</title>', 1)
	If @error Then Return $sLine
	Return $sFuncname[0]
EndFunc   ;==>GetFuncName

Func GetFileName()
	Local $sLine
	$g_sFNAME = ""
	While 1
		$sLine = FileReadLine($g_hIn)
		If @error Then Return SetError(@error, 0, 0) ; invalid TOC.hhc
		If StringInStr($sLine, '<param name=') Then
			; a StringRegExpReplace would be better ...
			If StringInStr($sLine, '"Local"') Then
				$g_sFNAME = StringReplace($sLine, '<param name="Local" value="' & $g_sOUTPUT_DIR, '', 1)
				$g_sFNAME = StringReplace($g_sFNAME, '">', '', 1)
				$g_sFNAME = StringStripWS($g_sFNAME, BitOR($STR_STRIPLEADING, $STR_STRIPTRAILING))
			Else
				$g_sNAME = StringReplace($sLine, '<param name="Name" value="', '', 1)
				$g_sNAME = StringReplace($g_sNAME, '">', '', 1)
				$g_sNAME = StringReplace($g_sNAME, ' Management', '', 1)
				$g_sNAME = StringStripWS($g_sNAME, BitOR($STR_STRIPLEADING, $STR_STRIPTRAILING))
			EndIf

		Else
			; skip until end  object
			If StringInStr($sLine, '</OBJECT>') Then Return 1
			If StringInStr($sLine, '</UL>') Then
				$g_sFNAME = "</UL>"
				Return 1
			EndIf
		EndIf
	WEnd
EndFunc   ;==>GetFileName

Func GetFirstManagement()
	Local $sLine

	; position on the top level as "Function Reference" or "User Defined Functions" or "Methods Reference"
	While 1
		$sLine = FileReadLine($g_hIn)
		If @error Then Return SetError(@error, 0, 0) ; invalid TOC.hhc
		If StringInStr($sLine, '<param name="Name" value="' & $g_sRefTypeS & " Reference") Then ExitLoop
	WEnd

	While @error = 0 And StringInStr($g_sFNAME, " Management") = 0
		GetFileName()
	WEnd

	If @error Then Return SetError(@error, 0, 0)
	$g_sFTOC = $g_sFNAME
	Return 1
EndFunc   ;==>GetFirstManagement

Func GetNextManagement()
	If $g_iReGen_UDFs Then
		While @error = 0 And StringInStr($g_sFNAME, " Management") = 0
			GetFileName()
		WEnd

		If @error Then Return SetError(@error, 0, 0)
		$g_sFTOC = $g_sFNAME
		Return 1
	EndIf

	If StringInStr($g_sFNAME, " Management") = 0 Then GetFileName()
	$g_sFTOC = $g_sFNAME
	Return StringInStr($g_sFNAME, " Management") Or StringInStr($g_sNAME, " Reference")
EndFunc   ;==>GetNextManagement

Func GetNextEntry()
	GetFileName()

	If StringInStr($g_sFNAME, " Management") And Not $g_iReGen_UDFs Then
		; ignore previous entry as it is a sublevel has been defined
		$g_sFTOC = $g_sFNAME
		genFiles()
	EndIf

	If StringInStr($g_sFNAME, "</UL>") Then
		Return 0
	EndIf

	Return 1
EndFunc   ;==>GetNextEntry

Func genFiles()
	Do
		$g_hOut = FileOpen($g_sOUTPUT_DIR & StringReplace($g_sFTOC, "&amp;", "&"), BitOR($FO_OVERWRITE, $FO_UTF8))
		$g_iNestedMgt += 1
		$g_ahOut[$g_iNestedMgt] = $g_hOut
		$g_anPrevFuncname_0[$g_iNestedMgt] = $g_aFuncname[0] + 1
		putHeader()
		While GetNextEntry()
			PutEntry()
		WEnd

		_ArraySort($g_aFuncname, 0, $g_anPrevFuncname_0[$g_iNestedMgt], $g_aFuncname[0])
		Local $sPrevFuncname = "", $sFuncname, $sIncludeName
		For $i = $g_anPrevFuncname_0[$g_iNestedMgt] To $g_aFuncname[0]
			$g_sFNAME = $g_aFuncname[$i]
			; write CatEntries
			$sFuncname = GetFuncName($g_sFNAME)
			If $sPrevFuncname <> $sFuncname Then ; avoid identical entries
				$sPrevFuncname = $sFuncname

				; Flag entry with mix include (true only in WinAPIEx Reference entries
				$sIncludeName = GetIncludeName($g_sFNAME)
				If $sIncludeName = "WinAPI.au3" And StringInStr($g_sFTOC, "WinAPIEx") Then
					$sIncludeName = " (*)"
				Else
					$sIncludeName = ""
				EndIf

				put('<tr>')
				put('<td><a href="' & $g_sFNAME & '">' & $sFuncname & '</a>' & $sIncludeName & '</td>')
				put('<td>' & GetFuncDesc($g_sFNAME) & '</td>')
				put('</tr>')
			EndIf
		Next

		put('</table>')
		put('')
		put('</body>')
		FileClose($g_hOut)
		$g_aFuncname[0] = $g_anPrevFuncname_0[$g_iNestedMgt] - 1
		$g_iNestedMgt -= 1
		$g_hOut = $g_ahOut[$g_iNestedMgt]
	Until GetNextManagement() = 0

	Return
EndFunc   ;==>genFiles

;------------------------------------------------------------------------------
; Write a new line to the output file
;------------------------------------------------------------------------------
Func put($sLineToWrite)
	FileWriteLine($g_hOut, $sLineToWrite)
EndFunc   ;==>put
