; ===================================================================
; Project: Macro consolidation
; Description: Creates a valid HTML5 macro file.
; Author: guinness
; ===================================================================

#include <File.au3>

#include '..\..\_build\include\MiscLib.au3'
#include '..\..\_build\include\OutputLib.au3'

FileChangeDir(@ScriptDir) ; As the build scripts have a habit of changing the working directory.

Opt('TrayIconDebug', 1)

OnAutoItExitRegister('OnQuit') ; ### When run in standalone
_OutputWindowCreate() ; ### When run in standalone

_OutputProgressWrite('macros.htm creation...')

GenerateMacros(@ScriptDir & '\html\macros', @ScriptDir & '\html\macros.htm')

_OutputProgressWrite('Complete.' & @CRLF)

Func OnQuit()
	_OutputWaitClosed() ; ### When run in standalone
EndFunc   ;==>OnQuit

Func GenerateMacros($sMacroPath, $sHTMLOutPath)
	Local $aFileList = _FileListToArray($sMacroPath, '*.htm')
	If @error Then
		_OutputProgressWrite('Missing macro files - please re-generate the html files for the individual macros.' & @CRLF)
		Return False
	EndIf

	Local $sData = ''
	; Read the macro files.
	For $i = 1 To UBound($aFileList) - 1
		$sData &= FileRead($sMacroPath & '\' & $aFileList[$i])
		If @error Then
			_OutputProgressWrite('There was an error reading ' & $aFileList[$i] & '.' & @CRLF)
			Return False
		EndIf
	Next

	Local $aMacroList = 0, _
			$hMacroList = 0
	; Generate a 2d array of macros.
	_MacroListToArray($sData, $aMacroList, $hMacroList)

	; HTML header.
	Local $sHTML = '<!DOCTYPE html>' & @CRLF
	$sHTML &= '<html>' & @CRLF
	$sHTML &= '<head>' & @CRLF
	$sHTML &= @TAB & '<title>Macros</title>' & @CRLF
	$sHTML &= @TAB & '<meta charset="utf-8">' & @CRLF
	$sHTML &= @TAB & '<link href="css/default.css" rel="stylesheet">' & @CRLF
	$sHTML &= '</head>' & @CRLF
	$sHTML &= '<body>' & @CRLF
	$sHTML &= @TAB & '<h1>Macro Reference</h1>' & @CRLF
	$sHTML &= @TAB & '<p>Below is an alphabetized list of all the macros available in AutoIt.<br /></p>' & @CRLF
	$sHTML &= @TAB & '<p>&nbsp;</p>' & @CRLF
	$sHTML &= @TAB & '<table>' & @CRLF
	$sHTML &= @TAB & '<tr>' & @CRLF
	$sHTML &= @TAB & @TAB & '<th>Macro</th>' & @CRLF
	$sHTML &= @TAB & @TAB & '<th>Description</th>' & @CRLF
	$sHTML &= @TAB & '</tr>' & @CRLF

	For $i = 0 To UBound($aMacroList) - 1
		$sHTML &= @TAB & '<tr>' & @CRLF
		$sHTML &= @TAB & @TAB & '<td><a id="' & $aMacroList[$i][$MACROLISTTOARRAY_NAME] & '"></a><strong>' & $aMacroList[$i][$MACROLISTTOARRAY_NAME] & '</strong></td>' & @CRLF
		$sHTML &= @TAB & @TAB & '<td>' & StringReplace($aMacroList[$i][$MACROLISTTOARRAY_DESCRIPTION], '<a href="../', '<a href="') & '</td>' & @CRLF
		$sHTML &= @TAB & '</tr>' & @CRLF
	Next

	$sHTML &= @TAB & '</table>' & @CRLF
	$sHTML &= '</body>' & @CRLF
	$sHTML &= '</html>' & @CRLF

	If Not _FileWrite($sHTMLOutPath, $sHTML, $FO_UTF8) Then
		_OutputProgressWrite('There was an error creating the macros.htm file.' & @CRLF)
		Return False
	EndIf

	Return True
EndFunc   ;==>GenerateMacros
